use serde::Serialize;
use std::ffi::OsStr;
use std::fmt;
use std::fs;

#[derive(Debug, Clone, Serialize)]
pub struct SoundClip {
    hash: String,
    file_name: String,
    directory: String,
}

impl fmt::Display for SoundClip {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(
            f,
            "SoundClip (hash: {}, file_name: {})",
            self.hash, self.file_name
        )
    }
}

impl SoundClip {
    pub fn new(file_name: String, directory: String) -> Self {
        Self {
            hash: encode_filename(&file_name),
            file_name: file_name,
            directory: directory,
        }
    }

    pub fn hash(&self) -> &String {
        &self.hash
    }

    pub fn file_name(&self) -> &String {
        &self.file_name
    }

    pub fn full_file_name(&self) -> String {
        format!("{}/{}", self.directory, self.file_name)
    }
}

fn encode_filename(file_name: &str) -> String {
    let hash = xxhash_rust::xxh3::xxh3_64(file_name.as_bytes());
    let hash_hex = format!("{:x}", hash);
    hash_hex
}

pub fn scan_directory_for_clips(directory: &str, extensions: &[&str]) -> Option<Vec<SoundClip>> {
    let mut sound_clips = Vec::new();

    if let Ok(entries) = fs::read_dir(directory) {
        for entry in entries {
            if let Ok(entry) = entry {
                let path = entry.path();
                if path.is_file() {
                    if let Some(ext) = path.extension() {
                        if extensions.iter().any(|&e| ext == OsStr::new(e)) {
                            if let Some(file_name) = path.file_name().unwrap().to_str() {
                                /*
                                the calls to unwrap() are more or less "safe" because:

                                file_name() only returns None if the path ends in "..", which won't be the case for paths returned by read_dir.
                                to_str() only returns None if the file name isn't valid Unicode, which is quite rare on most modern file systems.
                                 */
                                let sound_clip =
                                    SoundClip::new(file_name.to_string(), directory.to_owned());
                                sound_clips.push(sound_clip);
                            }
                        }
                    }
                }
            } else {
                println!("Failed to read directory entry: {}", entry.unwrap_err());
            }
        }
    } else {
        println!(
            "Failed to read directory: {}",
            fs::read_dir(directory).unwrap_err()
        );
        return None;
    }

    Some(sound_clips)
}
